@@ -21,7 +21,7 @@
 #'  separately. The reason why \code{same_N=TRUE} by default is that taking 
 #'  \code{N_opt} according to \code{alpha} could provide crossing conditional 
 #'  quantile curves (rarely observed for not too close values of \code{alpha}). 
-#'  The function \code{\link{plot.select.N.QuantifQuantile}} 
+#'  The function \code{\link{plot.QuantifQuantile}} 
 #'  illustrates the selection of \code{N_opt}. If the graph is not decreasing 
 #'  then increasing, the argument \code{testN} should be adapted.}
 #'  
@@ -35,12 +35,15 @@
 #' @param tildeB number of bootstrap replications for the choice of \code{N}.
 #' @param same_N whether to use the same value of \code{N} for each \code{alpha}
 #' (\code{TRUE} by default).
+#' @param ncores number of cores to use. Default is set to the number of cores 
+#' detected by R minus 1.
 #'  
 #' @return An object of class \code{QuantifQuantile} which is a list with the 
 #' following components:
 #' @return \item{hatq_opt}{A matrix containing the estimated conditional 
 #' quantiles. The number of columns is the number of considered values for \code{x} 
-#' and the number of rows the size of the order vector \code{alpha}.}
+#' and the number of rows the size of the order vector \code{alpha}. This object 
+#'  can also be returned using the usual \code{fitted.values} function.}
 #' @return \item{N_opt}{Optimal selected value for \code{N}. An integer if 
 #' \code{same_N=TRUE} and a vector of integers of length \code{length(alpha)} 
 #' otherwise.}
@@ -58,34 +61,33 @@
 
 #' @references Charlier, I. and Paindaveine, D. and Saracco, J.,
 #' \emph{Conditional quantile estimation through optimal quantization}, 
-#' Journal of Statistical Planning and Inference, to appear.
+#' Journal of Statistical Planning and Inference, 2015 (156), 14-30.
 #' @references Charlier, I. and Paindaveine, D. and Saracco, J.,
 #' \emph{Conditional quantile estimator based on optimal 
 #' quantization: from theory to practice}, Submitted.
 #' 
 #' @seealso \code{\link{QuantifQuantile}} and \code{\link{QuantifQuantile.d}} 
 #' for other dimensions.
-#' @seealso \code{\link{plot.select.N.QuantifQuantile}} for the \code{N} 
-#' selection criterion.
 #' @seealso \code{\link{plot.QuantifQuantile}}, 
 #' \code{\link{print.QuantifQuantile}}, \code{\link{summary.QuantifQuantile}}
 #' 
 #' @examples
 #' \dontrun{
-#' #(a few minuts to execute)
-#' set.seed(253664)
+#' #(a few seconds to execute)
+#' set.seed(164964)
 #' n <- 1000
 #' X <- matrix(runif(n*2,-2,2),ncol=n)
 #' Y <- apply(X^2,2,sum)+rnorm(n)
 #' res <- QuantifQuantile.d2(X,Y,testN=seq(90,140,by=10),B=20,tildeB=15)
 #' res2 <- QuantifQuantile.d2(X,Y,testN=seq(90,150,by=10),B=20,tildeB=15,same_N=FALSE)
 #' }
+#' @import parallel
 
 QuantifQuantile.d2 <- function(X, Y, alpha = c(0.05, 0.25, 0.5, 
     0.75, 0.95), x = matrix(c(rep(seq(min(X[1, ]), max(X[1, ]), 
     length = 20), 20), sort(rep(seq(min(X[2, ]), max(X[2, ]), 
     length = 20), 20))), nrow = 2, byrow = TRUE), testN = c(110, 
-    120, 130, 140, 150), p = 2, B = 50, tildeB = 20, same_N=TRUE) {
+    120, 130, 140, 150), p = 2, B = 50, tildeB = 20, same_N=TRUE,ncores=detectCores()-1) {
     if (!is.numeric(X)) 
         stop("X must be numeric")
     if (!is.numeric(Y)) 
@@ -133,9 +135,7 @@ QuantifQuantile.d2 <- function(X, Y, alpha = c(0.05, 0.25, 0.5,
     primeX <- array(X[, sample(c(1:n), n * (B + tildeB), replace = T)], 
         dim = c(d, n, (B + tildeB)))
     
-    for (jj in 1:length(testN)) {
-        N <- testN[jj]
-        
+    calc_hatq_N <- function(N){  
         hatX <- choice.grid(X, N, ng = (B + tildeB) )$opti_grid
         
         # projection of the sample X on the B+tildeB optimal grids
@@ -230,17 +230,21 @@ QuantifQuantile.d2 <- function(X, Y, alpha = c(0.05, 0.25, 0.5,
         hatq <- apply(Hatq[, , c(1:B), drop = FALSE], c(1, 2), 
             mean)
         
-        i <- which(N == testN)
-        hatq_N[, , i] <- t(hatq)
-        
         # the last tilde B are used to estimate the ISE
         HATq <- array(rep(hatq, tildeB), dim = c(m, length(alpha), 
             tildeB))
         hatISE <- (HATq - Hatq[, , c((1 + B):(B + tildeB)), drop = FALSE])^2
         hatISE <- apply(hatISE, 2, sum)/(m * tildeB)
-        hatISE_N[, i] <- hatISE
         
         print(N)
+        list(hatq=hatq,hatISE=hatISE)
+    }
+    
+    parallel_hatq_hatISE <- mclapply(testN,calc_hatq_N,mc.cores = ncores,mc.set.seed=F)
+    
+    for(i in 1:length(testN)){
+      hatq_N[,,i] <- t(parallel_hatq_hatISE[[i]]$hatq)
+      hatISE_N[,i] <- parallel_hatq_hatISE[[i]]$hatISE
     }
     
     if(same_N){
@@ -267,7 +271,7 @@ QuantifQuantile.d2 <- function(X, Y, alpha = c(0.05, 0.25, 0.5,
       }
     }
     
-    output <- list(hatq_opt = hatq_opt, N_opt = N_opt, 
+    output <- list(fitted.values = hatq_opt,hatq_opt = hatq_opt, N_opt = N_opt, 
         hatISE_N = hatISE_N, hatq_N = hatq_N, X = X, Y = Y, x = x, 
         alpha = alpha, testN = testN)
     class(output) <- "QuantifQuantile"
